@@ -1,4 +1,4 @@
 from .call import Call
 from .call_examples import call_examples
-from .parse import Parse
-from ._version import __version__
\ No newline at end of file
+from .parse import parse
+from ._version import __version__
