@@ -10,7 +10,6 @@ from concurrent.futures import ThreadPoolExecutor
 import os
 import logging
 from logging.handlers import TimedRotatingFileHandler
-import sys
 import shutil
 
 import sgex
@@ -29,6 +28,7 @@ stream_handler.setFormatter(formatter)
 logger.addHandler(file_handler)
 logger.addHandler(stream_handler)
 
+
 class Call:
     """Executes Sketch Engine API calls & saves data to desired output.
 
@@ -48,6 +48,8 @@ class Call:
 
     `threads` for asynchronous calling (`None` for default, otherwise an integer <= 32)
 
+    `halt` (`True`) abort calls on HTTP response error
+
     `loglevel` (`"warning"`) outputs to `.sgex.log`"""
 
     def _get_config(self):
@@ -69,7 +71,7 @@ class Call:
                     "noske": {
                         "server": "http://localhost:10070/bonito/run.cgi",
                         "asynchronous": True,
-                        },
+                    },
                     "ske": {
                         "api_key": "key",
                         "server": "https://api.sketchengine.eu/bonito/run.cgi",
@@ -240,7 +242,9 @@ class Call:
 
             # Process packet
             packet = {"item": manifest_item, "response": response}
-            logger.info(f'RESPONSE {packet["response"].status_code}: {packet["item"]["id"]}')
+            logger.info(
+                f'RESPONSE {packet["response"].status_code}: {packet["item"]["id"]}'
+            )
             self._post_call(packet)
 
             # Wait
@@ -264,32 +268,42 @@ class Call:
             logger.info(f'BATCH {manifest[x]["id"]}')
 
             with ThreadPoolExecutor(max_workers=THREAD_POOL) as executor:
-                for packet in list(executor.map(get, manifest[x:x+self.batch_size])):
+                for packet in list(
+                    executor.map(get, manifest[x : x + self.batch_size])
+                ):
                     # Process packets
-                    logger.debug(f'RESPONSE {packet["response"].status_code}: {packet["item"]["id"]}')
+                    logger.debug(
+                        f'RESPONSE {packet["response"].status_code}: {packet["item"]["id"]}'
+                    )
                     self._post_call(packet)
 
     def _post_call(self, packet):
         """Processes and saves call data."""
 
-        packet["response"].raise_for_status()
+        if self.halt:
+            packet["response"].raise_for_status()
 
         if self.format == "json":
-            response_json = packet["response"].json()
+            if packet["response"].status_code >= 400:
+                response_json = {
+                    "error": ": ".join(
+                        [str(packet["response"].status_code), packet["response"].reason]
+                    )
+                }
+            else:
+                response_json = packet["response"].json()
             # Keep data
             if "keep" in packet["item"]["params"]:
                 keep = packet["item"]["params"]["keep"]
                 if isinstance(keep, str):
-                    keeps = [keep]
+                    keeps = [keep, "error"]
                 elif isinstance(keep, (list, tuple)):
-                    keeps = keep
+                    keeps = tuple(keep) + tuple(["error"])
                 else:
                     raise TypeError(
                         f'Bad type for "keep" {type(keep)}: use string, list, tuple'
                     )
-                kept = {
-                    k: v for k, v in response_json.items() if k in keeps
-                }
+                kept = {k: v for k, v in response_json.items() if k in keeps}
                 self.data = kept
             else:
                 self.data = response_json
@@ -298,7 +312,7 @@ class Call:
                 for i in ["api_key", "username"]:
                     if i in self.data["request"]:
                         del self.data["request"][i]
-            
+
             # API errors
             error = None
             if "error" in response_json:
@@ -317,9 +331,9 @@ class Call:
                     meta = packet["item"]["params"]["meta"]
 
             versions = {
-               "api_version": response_json.get("api_version"),
-               "manatee_version": response_json.get("manatee_version"),
-               "sgex_version": sgex.__version__,
+                "api_version": response_json.get("api_version"),
+                "manatee_version": response_json.get("manatee_version"),
+                "sgex_version": sgex.__version__,
             }
 
             # Write to db
@@ -346,8 +360,8 @@ class Call:
             if not self.format == "json":
                 self.data = packet["response"]
             dir = pathlib.Path(self.output)
-            name = pathlib.Path(packet["item"]["id"]).with_suffix(self.extension)
-            self.file = dir / str(name)
+            name = pathlib.Path(str(packet["item"]["id"])).with_suffix(self.extension)
+            self.file = dir / name
             save_method = "".join(["_save_", self.format])
             getattr(Call, save_method)(self)
 
@@ -441,6 +455,7 @@ class Call:
         clear=False,
         server="ske",
         threads=None,
+        halt=True,
         loglevel="warning",
     ):
         # Settings
@@ -456,6 +471,7 @@ class Call:
         self.supported_formats = ["csv", "json", "txt", "xml", "xlsx"]
         self.config_file = None
         self.config_file_default = ".config.yml"
+        self.halt = halt
         if isinstance(input, str):
             self.input = pathlib.Path(input).name
         if threads:
@@ -474,7 +490,7 @@ class Call:
         config = self._get_config()
         server_info = self._get_server_info(server, config)
         self.server = server_info.get("server").strip("/")
-        self.calls = sgex.Parse(input).calls
+        self.calls = sgex.parse(input)
         self._reuse_parameters()
         self._set_wait(server_info)
         credentials = {
@@ -537,11 +553,11 @@ class Call:
             self.conn.close()
 
         if self.errors:
-            logger.info(f"{len(self.errors)} API error(s): {set(self.errors)}")
-        
+            logger.info(f"{len(self.errors)} error(s): {set(self.errors)}")
+
         t1 = time.perf_counter()
         logger.info(f"CALLED {self.called} in {t1 - t0:0.2f} secs")
 
         if self.dry_run:
             print("\n\nDRY RUN")
-            [print(f"{k}{v}") for k, v in self.summary().items()]
\ No newline at end of file
+            [print(f"{k}{v}") for k, v in self.summary().items()]
